@@ -4,8 +4,11 @@ from .collections import FolderCollection
 from .known_folders import (
     NON_DELETABLE_FOLDERS,
     AdminAuditLogs,
+    AllCategorizedItems,
     AllContacts,
     AllItems,
+    AllPersonMetadata,
+    AllTodoTasks,
     ApplicationData,
     ArchiveDeletedItems,
     ArchiveInbox,
@@ -18,6 +21,7 @@ from .known_folders import (
     Birthdays,
     Calendar,
     CalendarLogging,
+    CalendarSearchCache,
     CommonViews,
     Companies,
     Conflicts,
@@ -31,16 +35,21 @@ from .known_folders import (
     Directory,
     DlpPolicyEvaluation,
     Drafts,
+    EventCheckPoints,
     ExchangeSyncData,
+    ExternalContacts,
     Favorites,
     Files,
+    FolderMemberships,
     FreeBusyCache,
     FreebusyData,
     Friends,
+    FromFavoriteSenders,
     GALContacts,
     GraphAnalytics,
     IMContactList,
     Inbox,
+    Inference,
     Journal,
     JunkEmail,
     LocalFailures,
@@ -50,8 +59,9 @@ from .known_folders import (
     MsgFolderRoot,
     MyContacts,
     MyContactsExtended,
-    NonDeletableFolderMixIn,
+    NonDeletableFolder,
     Notes,
+    OneNotePagePreviews,
     OrganizationalContacts,
     Outbox,
     ParkedMessages,
@@ -59,21 +69,32 @@ from .known_folders import (
     PdpProfileV2Secured,
     PeopleCentricConversationBuddies,
     PeopleConnect,
+    PersonMetadata,
+    QedcDefaultRetention,
+    QedcLongRetention,
+    QedcMediumRetention,
+    QedcShortRetention,
+    QuarantinedEmail,
+    QuarantinedEmailDefaultCategory,
     QuickContacts,
     RecipientCache,
     RecoverableItemsDeletions,
     RecoverableItemsPurges,
     RecoverableItemsRoot,
+    RecoverableItemsSubstrateHolds,
     RecoverableItemsVersions,
     RecoveryPoints,
+    RelevantContacts,
     Reminders,
     RSSFeeds,
     Schedule,
     SearchFolders,
     SentItems,
     ServerFailures,
+    SharePointNotifications,
     Sharing,
     Shortcuts,
+    ShortNotes,
     Signal,
     SkypeTeamsMessages,
     SmsAndChatsSync,
@@ -81,9 +102,11 @@ from .known_folders import (
     SwssItems,
     SyncIssues,
     System,
+    System1,
     Tasks,
     TemporarySaves,
     ToDoSearch,
+    UserCuratedContacts,
     Views,
     VoiceMail,
     WellknownFolder,
@@ -94,8 +117,11 @@ from .roots import ArchiveRoot, PublicFoldersRoot, Root, RootOfHierarchy
 
 __all__ = [
     "AdminAuditLogs",
+    "AllCategorizedItems",
     "AllContacts",
     "AllItems",
+    "AllPersonMetadata",
+    "AllTodoTasks",
     "ApplicationData",
     "ArchiveDeletedItems",
     "ArchiveInbox",
@@ -110,6 +136,7 @@ __all__ = [
     "Birthdays",
     "Calendar",
     "CalendarLogging",
+    "CalendarSearchCache",
     "CommonViews",
     "Companies",
     "Conflicts",
@@ -125,21 +152,26 @@ __all__ = [
     "DistinguishedFolderId",
     "DlpPolicyEvaluation",
     "Drafts",
+    "EventCheckPoints",
     "ExchangeSyncData",
+    "ExternalContacts",
     "FOLDER_TRAVERSAL_CHOICES",
     "Favorites",
     "Files",
     "Folder",
     "FolderCollection",
     "FolderId",
+    "FolderMemberships",
     "FolderQuerySet",
     "FreeBusyCache",
     "FreebusyData",
     "Friends",
+    "FromFavoriteSenders",
     "GALContacts",
     "GraphAnalytics",
     "IMContactList",
     "Inbox",
+    "Inference",
     "Journal",
     "JunkEmail",
     "LocalFailures",
@@ -150,8 +182,9 @@ __all__ = [
     "MyContacts",
     "MyContactsExtended",
     "NON_DELETABLE_FOLDERS",
-    "NonDeletableFolderMixIn",
+    "NonDeletableFolder",
     "Notes",
+    "OneNotePagePreviews",
     "OrganizationalContacts",
     "Outbox",
     "ParkedMessages",
@@ -159,15 +192,24 @@ __all__ = [
     "PdpProfileV2Secured",
     "PeopleCentricConversationBuddies",
     "PeopleConnect",
+    "PersonMetadata",
     "PublicFoldersRoot",
+    "QedcDefaultRetention",
+    "QedcLongRetention",
+    "QedcMediumRetention",
+    "QedcShortRetention",
+    "QuarantinedEmail",
+    "QuarantinedEmailDefaultCategory",
     "QuickContacts",
     "RSSFeeds",
     "RecipientCache",
     "RecoverableItemsDeletions",
     "RecoverableItemsPurges",
     "RecoverableItemsRoot",
+    "RecoverableItemsSubstrateHolds",
     "RecoverableItemsVersions",
     "RecoveryPoints",
+    "RelevantContacts",
     "Reminders",
     "Root",
     "RootOfHierarchy",
@@ -177,7 +219,9 @@ __all__ = [
     "SearchFolders",
     "SentItems",
     "ServerFailures",
+    "SharePointNotifications",
     "Sharing",
+    "ShortNotes",
     "Shortcuts",
     "Signal",
     "SingleFolderQuerySet",
@@ -187,9 +231,11 @@ __all__ = [
     "SwssItems",
     "SyncIssues",
     "System",
+    "System1",
     "Tasks",
     "TemporarySaves",
     "ToDoSearch",
+    "UserCuratedContacts",
     "Views",
     "VoiceMail",
     "WellknownFolder",
