@@ -231,6 +231,11 @@ class AutodiscoverTest(EWSTest):
         if not self.settings.get("client_id") or not self.settings.get("username"):
             self.skipTest("This test requires delegate OAuth setup")
 
+        self.skipTest(
+            "Currently throws this error: Due to a configuration change made by your administrator, or because "
+            "you moved to a new location, you must use multi-factor authentication to access '0000-aaa-bbb-0000'"
+        )
+
         credentials = OAuth2LegacyCredentials(
             client_id=self.settings["client_id"],
             client_secret=self.settings["client_secret"],
@@ -256,6 +261,14 @@ class AutodiscoverTest(EWSTest):
         ad.discover()
         p = autodiscover_cache[ad._cache_key]
 
+        # Test invalid settings
+        with self.assertRaises(ValueError) as e:
+            p.get_user_settings(user=None, settings=["XXX"])
+        self.assertIn(
+            "Setting 'XXX' is invalid. Valid options are:",
+            e.exception.args[0],
+        )
+
         # Test invalid email
         invalid_email = get_random_email()
         r = p.get_user_settings(user=invalid_email)
@@ -834,16 +847,22 @@ class AutodiscoverTest(EWSTest):
         self.assertTrue(a._redirect_url_is_valid(self.account.protocol.config.service_endpoint))
 
     def test_protocol_default_values(self):
-        # Test that retry_policy and auth_type always get a value regardless of how we create an Account
-        self.get_account()
-        a = Account(
-            self.account.primary_smtp_address,
-            autodiscover=True,
-            config=self.account.protocol.config,
-        )
-        self.assertIsNotNone(a.protocol.auth_type)
-        self.assertIsNotNone(a.protocol.retry_policy)
+        # Test that retry_policy, auth_type and max_connections always get values regardless of how we create an Account
+        _max_conn = self.account.protocol.config.max_connections
+        try:
+            self.account.protocol.config.max_connections = 3
+            a = Account(
+                self.account.primary_smtp_address,
+                autodiscover=True,
+                config=self.account.protocol.config,
+            )
+            self.assertIsNotNone(a.protocol.auth_type)
+            self.assertIsNotNone(a.protocol.retry_policy)
+            self.assertEqual(a.protocol._session_pool_maxsize, 3)
+        finally:
+            self.account.protocol.config.max_connections = _max_conn
 
         a = Account(self.account.primary_smtp_address, autodiscover=True, credentials=self.account.protocol.credentials)
         self.assertIsNotNone(a.protocol.auth_type)
         self.assertIsNotNone(a.protocol.retry_policy)
+        self.assertEqual(a.protocol._session_pool_maxsize, a.protocol.SESSION_POOLSIZE)
