@@ -1,10 +1,14 @@
 from contextlib import suppress
+from inspect import isclass
 from unittest.mock import Mock
 
 import requests_mock
 
+import exchangelib.folders
+import exchangelib.folders.known_folders
 from exchangelib.errors import (
     DoesNotExist,
+    ErrorCannotDeleteObject,
     ErrorCannotEmptyFolder,
     ErrorDeleteDistinguishedFolder,
     ErrorFolderExists,
@@ -18,13 +22,17 @@ from exchangelib.errors import (
 )
 from exchangelib.extended_properties import ExtendedProperty
 from exchangelib.folders import (
-    NON_DELETABLE_FOLDERS,
     SHALLOW,
+    AllCategorizedItems,
     AllContacts,
     AllItems,
+    AllPersonMetadata,
+    AllTodoTasks,
     ApplicationData,
+    BaseFolder,
     Birthdays,
     Calendar,
+    CalendarSearchCache,
     CommonViews,
     Companies,
     Contacts,
@@ -35,13 +43,17 @@ from exchangelib.folders import (
     DistinguishedFolderId,
     DlpPolicyEvaluation,
     Drafts,
+    EventCheckPoints,
+    ExternalContacts,
     Favorites,
     Files,
     Folder,
     FolderCollection,
+    FolderMemberships,
     FolderQuerySet,
     FreeBusyCache,
     Friends,
+    FromFavoriteSenders,
     GALContacts,
     GraphAnalytics,
     IMContactList,
@@ -51,16 +63,19 @@ from exchangelib.folders import (
     Messages,
     MyContacts,
     MyContactsExtended,
+    NonDeletableFolder,
     Notes,
     OrganizationalContacts,
     Outbox,
     PassThroughSearchResults,
     PdpProfileV2Secured,
     PeopleCentricConversationBuddies,
+    PersonMetadata,
     PublicFoldersRoot,
     QuickContacts,
     RecipientCache,
     RecoveryPoints,
+    RelevantContacts,
     Reminders,
     RootOfHierarchy,
     RSSFeeds,
@@ -74,7 +89,15 @@ from exchangelib.folders import (
     SyncIssues,
     Tasks,
     ToDoSearch,
+    UserCuratedContacts,
     VoiceMail,
+    WellknownFolder,
+)
+from exchangelib.folders.known_folders import (
+    MISC_FOLDERS,
+    NON_DELETABLE_FOLDERS,
+    WELLKNOWN_FOLDERS_IN_ARCHIVE_ROOT,
+    WELLKNOWN_FOLDERS_IN_ROOT,
 )
 from exchangelib.items import Message
 from exchangelib.properties import CalendarPermission, EffectiveRights, InvalidField, Mailbox, PermissionSet, UserId
@@ -133,7 +156,7 @@ class FolderTest(EWSTest):
         self.account.root.remove_folder(Folder(id="XXX"))
         # Must be called on a distinguished folder class
         with self.assertRaises(ValueError):
-            RootOfHierarchy.get_distinguished(self.account)
+            RootOfHierarchy.get_distinguished(account=self.account)
         with self.assertRaises(ValueError):
             self.account.root.get_default_folder(Folder)
 
@@ -261,9 +284,10 @@ class FolderTest(EWSTest):
             ],
         )
         # Test top-level .children
-        self.assertListEqual(
-            [f.name for f in self.account.public_folders_root.children], ["Sample Contacts", "Sample Folder"]
-        )
+        children = list(self.account.public_folders_root.children)
+        self.assertListEqual([f.name for f in children], ["Sample Contacts", "Sample Folder"])
+        for f in children:
+            self.assertIsInstance(f.root, PublicFoldersRoot)
 
         find_public_subfolder1_children_xml = b"""\
 <?xml version="1.0" ?>
@@ -358,11 +382,12 @@ class FolderTest(EWSTest):
         # Test .get_children() on subfolders
         f_1 = self.account.public_folders_root / "Sample Contacts"
         f_2 = self.account.public_folders_root / "Sample Folder"
-        self.assertListEqual(
-            [f.name for f in self.account.public_folders_root.get_children(f_1)],
-            ["Sample Subfolder1", "Sample Subfolder2"],
-        )
-        self.assertListEqual([f.name for f in self.account.public_folders_root.get_children(f_2)], [])
+        f_1_children = list(self.account.public_folders_root.get_children(f_1))
+        self.assertListEqual([f.name for f in f_1_children], ["Sample Subfolder1", "Sample Subfolder2"])
+        for f in f_1_children:
+            self.assertIsInstance(f.root, PublicFoldersRoot)
+        f_2_children = list(self.account.public_folders_root.get_children(f_2))
+        self.assertListEqual([f.name for f in f_2_children], [])
 
     def test_invalid_deletefolder_args(self):
         with self.assertRaises(ValueError) as e:
@@ -460,54 +485,48 @@ class FolderTest(EWSTest):
         folders = list(FolderCollection(account=self.account, folders=[self.account.root]).get_folders())
         self.assertEqual(len(folders), 1, sorted(f.name for f in folders))
 
-        # Test that GetFolder can handle FolderId instances
-        folders = list(
-            FolderCollection(
-                account=self.account,
-                folders=[
-                    DistinguishedFolderId(
-                        id=Inbox.DISTINGUISHED_FOLDER_ID,
-                        mailbox=Mailbox(email_address=self.account.primary_smtp_address),
-                    )
-                ],
-            ).get_folders()
-        )
-        self.assertEqual(len(folders), 1, sorted(f.name for f in folders))
-
-    def test_get_folders_with_distinguished_id(self):
-        # Test that we return an Inbox instance and not a generic Messages or Folder instance when we call GetFolder
-        # with a DistinguishedFolderId instance with an ID of Inbox.DISTINGUISHED_FOLDER_ID.
-        inbox_folder_id = DistinguishedFolderId(
-            id=Inbox.DISTINGUISHED_FOLDER_ID, mailbox=Mailbox(email_address=self.account.primary_smtp_address)
-        )
-        inbox = list(
-            GetFolder(account=self.account).call(
-                folders=[inbox_folder_id],
-                shape="IdOnly",
-                additional_fields=[],
+        # Test that GetFolder cannot handle FolderId instances
+        with self.assertRaises(ValueError):
+            list(
+                FolderCollection(
+                    account=self.account,
+                    folders=[
+                        DistinguishedFolderId(
+                            id=Inbox.DISTINGUISHED_FOLDER_ID,
+                            mailbox=Mailbox(email_address=self.account.primary_smtp_address),
+                        )
+                    ],
+                ).get_folders()
             )
-        )[0]
-        self.assertIsInstance(inbox, Inbox)
-
-        # Test via SingleFolderQuerySet
-        inbox = SingleFolderQuerySet(account=self.account, folder=inbox_folder_id).resolve()
-        self.assertIsInstance(inbox, Inbox)
 
     def test_folder_grouping(self):
         # If you get errors here, you probably need to fill out [folder class].LOCALIZED_NAMES for your locale.
         for f in self.account.root.walk():
+            with self.subTest(f=f):
+                if f.is_distinguished:
+                    self.assertIsNotNone(f.DISTINGUISHED_FOLDER_ID)
+                else:
+                    self.assertIsNone(f.DISTINGUISHED_FOLDER_ID)
             with self.subTest(f=f):
                 if isinstance(
                     f,
                     (
-                        Messages,
-                        DeletedItems,
+                        AllCategorizedItems,
                         AllContacts,
-                        MyContactsExtended,
-                        Sharing,
+                        AllPersonMetadata,
+                        DeletedItems,
+                        Drafts,
                         Favorites,
-                        SyncIssues,
+                        FromFavoriteSenders,
+                        Inbox,
+                        JunkEmail,
+                        Messages,
                         MyContacts,
+                        MyContactsExtended,
+                        RelevantContacts,
+                        Sharing,
+                        SyncIssues,
+                        UserCuratedContacts,
                     ),
                 ):
                     self.assertEqual(f.folder_class, "IPF.Note")
@@ -523,6 +542,8 @@ class FolderTest(EWSTest):
                     self.assertEqual(f.folder_class, "IPF.StoreItem.RecoveryPoints")
                 elif isinstance(f, SwssItems):
                     self.assertEqual(f.folder_class, "IPF.StoreItem.SwssItems")
+                elif isinstance(f, EventCheckPoints):
+                    self.assertEqual(f.folder_class, "IPF.StoreItem.EventCheckPoints")
                 elif isinstance(f, PassThroughSearchResults):
                     self.assertEqual(f.folder_class, "IPF.StoreItem.PassThroughSearchResults")
                 elif isinstance(f, GraphAnalytics):
@@ -545,13 +566,13 @@ class FolderTest(EWSTest):
                     self.assertEqual(f.folder_class, "IPF.Contact.MOC.ImContactList")
                 elif isinstance(f, QuickContacts):
                     self.assertEqual(f.folder_class, "IPF.Contact.MOC.QuickContacts")
-                elif isinstance(f, Contacts):
+                elif isinstance(f, (Contacts, ExternalContacts)):
                     self.assertEqual(f.folder_class, "IPF.Contact")
                 elif isinstance(f, Birthdays):
                     self.assertEqual(f.folder_class, "IPF.Appointment.Birthday")
                 elif isinstance(f, Calendar):
                     self.assertEqual(f.folder_class, "IPF.Appointment")
-                elif isinstance(f, (Tasks, ToDoSearch)):
+                elif isinstance(f, (Tasks, ToDoSearch, AllTodoTasks, FolderMemberships)):
                     self.assertEqual(f.folder_class, "IPF.Task")
                 elif isinstance(f, Reminders):
                     self.assertEqual(f.folder_class, "Outlook.Reminder")
@@ -577,6 +598,10 @@ class FolderTest(EWSTest):
                     self.assertEqual(f.folder_class, "IPF.SkypeTeams.Message")
                 elif isinstance(f, SmsAndChatsSync):
                     self.assertEqual(f.folder_class, "IPF.SmsAndChatsSync")
+                elif isinstance(f, PersonMetadata):
+                    self.assertEqual(f.folder_class, "IPF.Contact")
+                elif isinstance(f, CalendarSearchCache):
+                    self.assertEqual(f.folder_class, "IPF.Appointment")
                 else:
                     self.assertIn(f.folder_class, (None, "IPF"), (f.name, f.__class__.__name__, f.folder_class))
                     self.assertIsInstance(f, Folder)
@@ -693,7 +718,7 @@ class FolderTest(EWSTest):
         folder = Folder()
         with self.assertRaises(ValueError):
             folder.refresh()  # Must have root folder
-        folder.root = self.account.root
+        folder._root = self.account.root
         with self.assertRaises(ValueError):
             folder.refresh()  # Must have an id
 
@@ -969,9 +994,66 @@ class FolderTest(EWSTest):
 
     def test_non_deletable_folders(self):
         for f in self.account.root.walk():
-            if f.__class__ not in NON_DELETABLE_FOLDERS:
+            with self.subTest(item=f):
+                if f.__class__ in NON_DELETABLE_FOLDERS + WELLKNOWN_FOLDERS_IN_ARCHIVE_ROOT + WELLKNOWN_FOLDERS_IN_ROOT:
+                    self.assertEqual(f.is_deletable, False)
+                    try:
+                        f.delete()
+                    except (ErrorDeleteDistinguishedFolder, ErrorCannotDeleteObject, ErrorItemNotFound):
+                        pass
+                else:
+                    self.assertEqual(f.is_deletable, True)
+                    # Don't attempt to delete. That could affect parallel tests
+
+    def test_folder_collections(self):
+        # Test that all custom folders are exposed in the top-level module
+        top_level_classes = [
+            cls
+            for cls in vars(exchangelib.folders).values()
+            if isclass(cls) and issubclass(cls, exchangelib.folders.BaseFolder)
+        ]
+        known_folder_classes = [
+            cls
+            for cls in vars(exchangelib.folders.known_folders).values()
+            if isclass(cls) and issubclass(cls, exchangelib.folders.BaseFolder)
+        ]
+        for cls in known_folder_classes:
+            with self.subTest(item=cls):
+                self.assertIn(cls, top_level_classes)
+
+        # Test that all custom folders are in one of the following folder collections
+        all_cls = NON_DELETABLE_FOLDERS + WELLKNOWN_FOLDERS_IN_ARCHIVE_ROOT + WELLKNOWN_FOLDERS_IN_ROOT + MISC_FOLDERS
+        for cls in top_level_classes:
+            if not isclass(cls) or not issubclass(cls, BaseFolder):
                 continue
-            self.assertEqual(f.is_deletable, False)
+            with self.subTest(item=cls):
+                if cls in NON_DELETABLE_FOLDERS + [NonDeletableFolder]:
+                    self.assertTrue(issubclass(cls, NonDeletableFolder))
+                elif cls in WELLKNOWN_FOLDERS_IN_ARCHIVE_ROOT + WELLKNOWN_FOLDERS_IN_ROOT + [WellknownFolder, Messages]:
+                    self.assertTrue(issubclass(cls, WellknownFolder))
+                else:
+                    self.assertFalse(issubclass(cls, WellknownFolder))
+                    self.assertFalse(issubclass(cls, NonDeletableFolder))
+            with self.subTest(item=cls):
+                if cls in WELLKNOWN_FOLDERS_IN_ARCHIVE_ROOT + WELLKNOWN_FOLDERS_IN_ROOT:
+                    self.assertIsNotNone(cls.DISTINGUISHED_FOLDER_ID)
+                elif cls in (BaseFolder, Folder, WellknownFolder, RootOfHierarchy):
+                    self.assertIsNone(cls.DISTINGUISHED_FOLDER_ID)
+                elif issubclass(cls, RootOfHierarchy):
+                    self.assertIsNotNone(cls.DISTINGUISHED_FOLDER_ID)
+                else:
+                    self.assertIsNone(cls.DISTINGUISHED_FOLDER_ID)
+            with self.subTest(item=cls):
+                if issubclass(cls, RootOfHierarchy) or cls in (
+                    BaseFolder,
+                    Folder,
+                    WellknownFolder,
+                    Messages,
+                    NonDeletableFolder,
+                ):
+                    self.assertNotIn(cls, all_cls)
+                else:
+                    self.assertIn(cls, all_cls)
 
     def test_folder_query_set(self):
         # Create a folder hierarchy and test a folder queryset
